/* vim:set ts=4 sw=4:
 *
 * (C) 2008 Willem Hengeveld  itsme@xs4all.nl
 *
 * IdcPerl - a perl scripting plugin for the Interactive Disassembler by hex-rays.
 * see http://www.xs4all.nl/~itsme/projects/idcperl
 *
 * this module contains code that dynamically imports all functions from DYNAMIC_PERL_DLL
 *
 */

#include "perldll.h"
struct PerlFuncDef {
    char* name;
    FARPROC* ptr;
};
// ida includes
#include "pro.h"     // for basic types.
#include "ida.hpp"   // for ida constants and types.
#include "idp.hpp"   // for interface version
#include "netnode.hpp"  // for RootNode
#include "expr.hpp"  // for IDCFuncs


// perl includes
#include "extern.h"
#include "perl.h"
#include "xsub.h"

#include "redefperlasdll.h"

#ifdef TRACE_DLL
#define tracemsg msg
#else
#define tracemsg  while(0)
#endif

// empty def, this is the module defining the function ptrs.
#define PERLDEFINE
#include "perldllprocs.h"

static PerlFuncDef perl_funcname_table[] = {
#include "perldllprocnames.inc"
};

int PerlDll::load_procs()
{
    for (PerlFuncDef *p=perl_funcname_table ; p->ptr ; p++)
    {
        *p->ptr = GetProcAddress(_h, p->name);
        if (*p->ptr==NULL)
        {
            msg("IDAPERL ERROR loading proc %s\n", p->name);
            return false;
        }
    }
    return true;
}
void PerlDll::unload()
{
    tracemsg("PerlDll::unload\n");
    if (_h) {
        FreeLibrary(_h);
        _h = NULL;
    }
}
int PerlDll::load()
{
    tracemsg("PerlDll::load\n");
    _h= LoadLibrary(DYNAMIC_PERL_DLL);
    if (_h==NULL || _h==INVALID_HANDLE_VALUE) {
        msg("IDAPERL ERROR: LoadLibrary " DYNAMIC_PERL_DLL);
        return false;
    }
    if (!load_procs()) {
        unload();
        return false;
    }
    tracemsg("PerlDll::load done\n");
    return true;
}

